package org.latexlab.clsi.client;

import org.latexlab.clsi.client.async.CompileCallback;
import org.latexlab.clsi.client.protocol.ClsiResourceReference;

import com.google.gwt.xml.client.XMLParser;

public abstract class ClsiService {

  protected int instance = 0;
  protected String token, id;
  
  public ClsiService() { }
  
  public ClsiService(String token, String id) {
    this.token = token;
    this.id = id;
  }
  
  public abstract void compile(String name, String contents, String primaryResource,
	      ClsiResourceReference[] resources, String compiler, String format,
		  CompileCallback callback);
	
  public String getId() {
    return id;
  }

  public void setId(String id) {
    this.id = id;
  }
  
  public String getToken() {
    return token;
  }

  public void setToken(String token) {
    this.token = token;
  }
  
  protected String buildCompileRequest(String name, String contents,
		String primaryResource, ClsiResourceReference[] resources,
		String compiler, String format) {
	com.google.gwt.xml.client.Document doc = XMLParser.createDocument();
	com.google.gwt.xml.client.Element compileNode = doc.createElement("compile");
	com.google.gwt.xml.client.Element tokenNode = doc.createElement("token");
	com.google.gwt.xml.client.Element idNode = doc.createElement("id");
	com.google.gwt.xml.client.Element instanceNode = doc.createElement("instance");
	com.google.gwt.xml.client.Element optionsNode = doc.createElement("options");
	com.google.gwt.xml.client.Element compilerNode = doc.createElement("compiler");
	com.google.gwt.xml.client.Element outputFormatNode = doc.createElement("output-format");
	com.google.gwt.xml.client.Element resourcesNode = doc.createElement("resources");
	com.google.gwt.xml.client.Element resourceNode;
	com.google.gwt.xml.client.Element literalResourceNode = doc.createElement("resource");
	com.google.gwt.xml.client.Element handlerNode = doc.createElement("handler-url");
	com.google.gwt.xml.client.Element forceUtf8Node = doc.createElement("force-utf8");
	tokenNode.appendChild(doc.createTextNode(this.token));
	idNode.appendChild(doc.createTextNode(this.id));
	instanceNode.appendChild(doc.createTextNode(String.valueOf(this.instance)));
	compilerNode.appendChild(doc.createTextNode(compiler));
	outputFormatNode.appendChild(doc.createTextNode(format));
	forceUtf8Node.appendChild(doc.createTextNode("\u8482"));
	resourcesNode.setAttribute("root-resource-path", primaryResource);
	literalResourceNode.setAttribute("path", name);
	literalResourceNode.appendChild(doc.createTextNode(contents));
	resourcesNode.appendChild(literalResourceNode);
	for (ClsiResourceReference ref : resources) {
	  resourceNode = doc.createElement("resource");
	  resourceNode.setAttribute("path", ref.getName());
	  resourceNode.setAttribute("modified", ref.getModified().toString());
	  resourceNode.setAttribute("url", ref.getUrl());
	  String mimeType = ref.getMimeType();
	  if (mimeType != null && !mimeType.equals("") ) {
		resourceNode.setAttribute("mimeType", mimeType);
	  }
	  String encoding = ref.getEncoding();
	  if (encoding != null && !encoding.equals("") ) {
		resourceNode.setAttribute("encoding", encoding);
	  }
	  String auth = ref.getAuthorization();
	  if (auth != null && !auth.equals("") ) {
		resourceNode.setAttribute("auth", auth);
	  }
	  resourcesNode.appendChild(resourceNode);
	}
	optionsNode.appendChild(compilerNode);
	optionsNode.appendChild(outputFormatNode);
	optionsNode.appendChild(forceUtf8Node);
	compileNode.appendChild(tokenNode);
	compileNode.appendChild(idNode);
	compileNode.appendChild(instanceNode);
	String handlerUrl = null;
	if (handlerUrl != null && !handlerUrl.equals("")) {
	  handlerNode.appendChild(doc.createTextNode(String.valueOf(handlerUrl)));
	  compileNode.appendChild(handlerNode);
	}
	compileNode.appendChild(optionsNode);
	compileNode.appendChild(resourcesNode);
	doc.appendChild(compileNode);
	String xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + doc.toString();
	return xml;
  }
}
